from __future__ import unicode_literals

import logging
import os

from mopidy import config, ext

__version__ = '2.0.2'

logger = logging.getLogger(__name__)

class Extension(ext.Extension):

    dist_name = 'Mopidy-Radio-Rough'
    ext_name = 'radio_rough'
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(Extension, self).get_config_schema()
        return schema

    def setup(self, registry):
        from .mopidy_radio_rough import MopidyRadioRough
        registry.add('frontend', MopidyRadioRough)


